/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <qapplication.h>
#include <qtranslator.h>
#include <qtextcodec.h>
#include <qmessagebox.h>

#include <errno.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <stdlib.h>

#include "kadu.h"
#include "kadu-config.h"

#include "config_file.h"
#include "config_dialog.h"
#include "register.h"
#include "misc.h"
#include "debug.h"
#include "modules.h"
#include "emoticons.h"

Kadu *kadu;	

#ifdef SIG_HANDLING_ENABLED
#include <qdatetime.h>
#include <signal.h>
#ifdef HAVE_EXECINFO
#include <execinfo.h>
#endif
void kadu_signal_handler(int s)
{
	kdebugm(KDEBUG_PANIC, "kadu_signal_handler: %d\n", s);
	if(lockFile){ // moze sie wywalic praktycznie po wylaczeniu i to tez trzeba uwzglednic	
		flock(lockFileHandle, LOCK_UN);
		kdebugm(KDEBUG_PANIC, "lock released\n");
		lockFile->close();
		kdebugm(KDEBUG_PANIC, "lockfile closed\n");
	}
	
	QString f=QString("kadu.conf.backup.%1").arg(QDateTime::currentDateTime().toString("yyyy.MM.dd.hh.mm.ss"));
	if (s==SIGSEGV)
	{
		kdebugm(KDEBUG_PANIC, "Kadu crashed :(\n");
#ifdef HAVE_EXECINFO
		void *bt_array[100];
		char **bt_strings;
		int num_entries;
		if ((num_entries = backtrace(bt_array, 100)) < 0) {
			kdebugm(KDEBUG_PANIC, "could not generate backtrace\n");
			return;
		}
		if ((bt_strings = backtrace_symbols(bt_array, num_entries)) == NULL) {
			kdebugm(KDEBUG_PANIC, "could not get symbol names for backtrace\n");
			return;
		}
		fprintf(stderr, "\n======= BEGIN OF BACKTRACE =====\n");
		for (int i = 0; i < num_entries; i++) {
			fprintf(stderr, "[%d] %s\n", i, bt_strings[i]);
		}
		fprintf(stderr, "======= END OF BACKTRACE  ======\n");
		free(bt_strings);
#else
		kdebugm(KDEBUG_PANIC, "backtrace not available\n");		
#endif
		config_file.saveTo(ggPath(f.latin1()));
		abort();
	}
	else if (s==SIGINT || s==SIGTERM)
		config_file.saveTo(ggPath(f.latin1()));
	exit(0);
}
#endif

int main(int argc, char *argv[])
{
	gg_debug_level = 255;
	debug_mask=KDEBUG_ALL & ~KDEBUG_FUNCTION_END;
	char *d = getenv("DEBUG_MASK");
	if (d)
	{
		debug_mask=atol(d);
		gg_debug_level=debug_mask | ~255;
	}

#ifdef SIG_HANDLING_ENABLED
	signal(SIGSEGV, kadu_signal_handler);
	signal(SIGINT, kadu_signal_handler);
	signal(SIGTERM, kadu_signal_handler);
#endif

	dataPath("", argv[0]);
	emoticons=new EmoticonsManager();

	new QApplication(argc, argv);

	// ladowanie tlumaczenia
	QTranslator qt_qm(0);
	QString lang=config_file.readEntry("General", "Language", QTextCodec::locale());
	qt_qm.load(dataPath(QString("kadu/translations/qt_") + lang), ".");
	qApp->installTranslator(&qt_qm);
	QTranslator kadu_qm(0);
	kadu_qm.load(dataPath(QString("kadu/translations/kadu_") + lang), ".");
	qApp->installTranslator(&kadu_qm);

	lockFile=new QFile(ggPath("lock"));
	if (lockFile->open(IO_ReadWrite))
	{
		lockFileHandle=lockFile->handle();
		if (flock(lockFileHandle, LOCK_EX|LOCK_NB)!=0)
		{
			kdebugm(KDEBUG_WARNING, "flock: %s\n", strerror(errno));
			if (QMessageBox::warning(NULL, "Kadu lock",
				qApp->translate("@default", QT_TR_NOOP("Lock file in profile directory exists. Another Kadu probably running.")),
				qApp->translate("@default", QT_TR_NOOP("Force running Kadu (not recommended).")),
				qApp->translate("@default", QT_TR_NOOP("Quit.")), 0, 1)==1 )
			return 1;
		}
	}

	IconsManager::initModule();
	kadu = new Kadu(0, "Kadu");

	Kadu::InitModules();
	QPixmap pix = icons_manager.loadIcon("Offline");
	kadu->setIcon(pix);
	qApp->setMainWidget(kadu);

	ModulesManager::initModule();

	kadu->startupProcedure();

	QObject::connect(qApp, SIGNAL(aboutToQuit()), kadu, SLOT(quitApplication()));

	return qApp->exec();
}
